package https;


import static common.Log.COMM;
import static common.Log.LOG_ON;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.nio.ByteBuffer;
import java.nio.channels.Channels;
import java.nio.channels.ReadableByteChannel;


import javax.net.ssl.HttpsURLConnection;

import org.apache.log4j.Level;


/**
 * Connect to an hhtps server, and receive data.
 * @author Éric Lallet.
 *
 */
public class HttpsProtocol  {
	/**
	 * ByteBuffer size.
	 */
	private static final int BUFFSIZE = 4096;
	/**
	 * status of the connexion.
	 */
	private Status status = Status.NotConnected;
	/**
	 * channel connected to the https serveur.
	 */
	private ReadableByteChannel httpsChan;
	/**
	 * receiving ByteBuffer.
	 */
	private ByteBuffer readBuff;


	/**
	 * 
	 */
	public HttpsProtocol() {
		readBuff =  ByteBuffer.allocate(BUFFSIZE);

	}

	/**
	 * connect hhtpsChan to the hhtps server.
	 * @param urlString
	 *                 URL of the hhtps server.
	 * @throws IOException
	 *                 the exception thrown in case of IO problem.
	 */
	public void connectTo(final String urlString) throws IOException {
		if (status == Status.Connected) {
			System.out.println("Allready connected");
		} else {
			if (LOG_ON && COMM.isEnabledFor(Level.TRACE)) {
				COMM.trace("URL: " + urlString);
			}
			URL myUrl = new URL(urlString);
			HttpsURLConnection connection = (HttpsURLConnection) myUrl.openConnection();
			InputStream inputStream = connection.getInputStream();
			httpsChan = Channels.newChannel(inputStream);
			status = Status.Connected;
		}
	}


	/**
	 * read text data sent by the https server.
	 * @return
	 *         text data sent by the https server. 
	 * @throws IOException
	 *                  the exception thrown in case of IO problem.
	 */
	public String recvData() throws IOException {
		String texte = "";
		int readSize;

		if (status != Status.Connected) {
			return null;
		}

		do {
			readBuff.clear();
			readSize = httpsChan.read(readBuff);
			readBuff.flip();
			byte[] byteArray = new byte[readBuff.limit()];
			readBuff.get(byteArray, 0, readBuff.limit());
			texte = texte + new String(byteArray);
		} while (readSize > 0);

		this.close();
		if (LOG_ON && COMM.isEnabledFor(Level.TRACE)) {
			COMM.trace("http body: " + texte);
		}
		return texte;
	}


	/**
	 * 
	 * @throws IOException
	 *                   the exception thrown in case of IO problem.
	 */
	public void close() throws IOException {
		if (status != Status.NotConnected) {
			httpsChan.close();
			status = Status.NotConnected;
		}
	}

	/**
	 * 
	 * @return 
	 *         status of httpsProtocol.
	 */
	public Status getStatus() {
		return status;
	}

	/**
	 * 
	 * @return
	 *        channel connected to httpsProtocol.
	 */
	public ReadableByteChannel getHttpChan() {
		return httpsChan;
	}

}
