package json;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;

/**
 * This class has the same structure that the result json field in the response of the web service.
 * Needed for parsing the json message. 
 * @author Éric Lallet.
 *
 */
public class Result {
	/**
	 * category of the question. 1st json subfield of the result json field.
	 */
	private String category;
	/**
	 * type of the question. 2nd json subfield of the result json field.
	 * supposed to be "multiple" choices. 
	 */
	private String type;
	/**
	 * difficulty of the question. 3rd json subfield of the result json field.
	 */
	private String difficulty;
	/**
	 * text of the question. 4th json subfield of the result json field.
	 */
	private String question;
	/**
	 * text of the correct answer for the question. 5th json subfield of the result json field.
	 */
	private String correct_answer;
	/**
	 * array of texts for incorrect propositions.  6th json subfield of the result json field.
	 * supposed to contain 3 propositions.
	 */
	private String[] incorrect_answers;



	@Override
	public String toString() {

		String res =  "{\"category\":\"" + category + "\"," 
		+ "\"type\":\"" + type + "\"," 
		+ "\"difficulty\":\"" + difficulty + "\"," 
		+ "\"question\":\"" + question + "\"," 
		+ "\"correct_answer\":\"" + correct_answer + "\"," 
		+ "\"incorrect_answers\":\"";

		for (int ind = 0; ind < incorrect_answers.length - 1; ind++) {
			res = res + "\"" + incorrect_answers[ind] + "\",";
		}
		if (incorrect_answers.length > 0) {
			res = res + "\"" + incorrect_answers[incorrect_answers.length - 1] + "\""; 
		}
		res = res + "]}";


		return decode(res); 

	}

	/**
	 * 
	 * @param URLencoded 
	 *                  text in json message (supposed to be url3986 encoded).
	 * @return
	 *        UTF8 text version.
	 */
	private String decode(String URLencoded) {
		String res = URLencoded ;
		try {
			res = URLDecoder.decode(URLencoded, StandardCharsets.UTF_8.toString());
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		return res;
	}

	/**
	 * 
	 * @return
	 *         category of the question.
	 */
	public String getCategory() {
		return decode(category);
	}

	/**
	 * 
	 * @return
	 *         difficulty of the question.
	 */
	public String getDifficulty() {
		return decode(difficulty);
	}

	
	/**
	 * 
	 * @return
	 *         text of the question.
	 */
	public String getQuestion() {
		return decode(question);
	}

	/**
	 * 
	 * @return
	 *         correct answer.
	 */
	public String getCorrectAnswer() {
		return decode(correct_answer);
	}

	/**
	 * 
	 * @param num
	 *           position in the incorrect answers list.
	 * @return
	 *        incorrect answer at the num position.
	 */
	public String getIncorrectAnswer(final int num) {
		return decode(incorrect_answers[num]);
	}

}
