package json;

import java.io.Serializable;
import java.util.Arrays;

/**
 * 
 * @author Éric Lallet.
 *
 */
public class TrivialQuestion implements Serializable {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	/**
	 * number of choices for the propositions (included correct answer).
	 */
	public static final int NBANSWERS = 4; 
	/**
	 * category of the question.
	 */
	private String category;
	/**
	 * difficulty of the question.
	 */
	private String difficulty;
	/**
	 * text of the question.
	 */
	private String question;
	/**
	 * list of the propositions  (included correct answer). randomized order.
	 */
	private String[] answers;
	
	/**
	 * position of the correct answer in answers.
	 */
	private int correctAnswerNum;
	/**
	 * 
	 * @param category
	 *                category of the question.
	 * @param difficulty
	 *                difficulty of the question.
	 * @param question
	 *                text of the question.
	 * @param answers
	 *               list of the propositions  (included correct answer). randomized order.
	 * @param correctAnswerNum
	 *                position of the correct answer in answers.
	 * @throws IllegalArgumentException
	 *               the exception thrown if the answer parameter has not the correct length.
	 */
	public TrivialQuestion(final String category, final String difficulty, final String question, final String[] answers, final int correctAnswerNum) throws IllegalArgumentException {
		this.category = category;
		this.difficulty = difficulty;
		this.question = question;
		if (answers.length != NBANSWERS) {
			throw new IllegalArgumentException("the array of awnsers must contain " + NBANSWERS + " propositions");
		}
		this.answers = answers;
		this.correctAnswerNum = correctAnswerNum;
	}
	
	/**
	 * 
	 * @return
	 *        category of the question.
	 */
	public String getCategory() {
		return category;
	}
	/**
	 * 
	 * @return
	 *        difficulty of the question.
	 */
	public  String getDifficulty() {
		return difficulty;
	}
	/**
	 * 
	 * @return
	 *        text of the question.
	 */
	public String getQuestion() {
		return question;
	}
	
	/**
	 * 
	 * @param num
	 *           position in the list of propositions.
	 * @return
	 *        answer at the num position. 
	 * @throws IllegalArgumentException
	 *                 the exception thrown if the num parameter has not the correct value.
	 *                 
	 */
	public String getAnwser(final int num) throws IllegalArgumentException {
		if ((num < 0) || (num >= NBANSWERS)) {
			throw new IllegalArgumentException("correct values for num parameter: 0-" + NBANSWERS);
		}
		return answers[num];
	}
	
	/**
	 * 
	 * @return
	 *        position of the correct answer in answers
	 */
	public int getCorrectAnswerNum() {
		return this.correctAnswerNum;
	}
	
	@Override
	public String toString() {
		String res = "Catergory: " + category 
				+ "\nDifficulty: " + difficulty 
				+ "\nQuestion: " + question;
		
		char letter = 'A';
		for (int ind = 0; ind < answers.length; ind++) {
			res = res + "\n\t[" + letter + "]: " +  answers[ind];
			letter++;
		}
		return res;
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + Arrays.hashCode(answers);
		result = prime * result + ((category == null) ? 0 : category.hashCode());
		result = prime * result + ((difficulty == null) ? 0 : difficulty.hashCode());
		result = prime * result + ((question == null) ? 0 : question.hashCode());
		return result;
	}

	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		TrivialQuestion other = (TrivialQuestion) obj;
		if (!Arrays.equals(answers, other.answers)) {
			return false;
		}
		if (category == null) { 
			if (other.category != null) {
				return false;
			}
		} else if (!category.equals(other.category)) {
			return false;
		}
		if (difficulty == null) { 
			if (other.difficulty != null) {
				return false;
			}
		} else if (!difficulty.equals(other.difficulty)) {
			return false;
		}
		if (question == null) {
			if (other.question != null) {
				return false;
			}
		} else if (!question.equals(other.question)) {
			return false;
		}
		return true;
	}

}
