package trivial;

import static common.Log.COMM;
import static common.Log.LOG_ON;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;

import javax.swing.SwingUtilities;

import org.apache.log4j.Level;

import common.FullDuplexMsgWorker;
import common.ReadMessageStatus;
import interface_graphique.AjouterLigne;
import interface_graphique.InterfaceGraphique;
import json.TrivialQuestion;


/**
 * 
 * @author Éric Lallet.
 *
 */
public class ThreadClient implements Runnable {
	/**
	 * worker pour la communication TCP avec la serveur.
	 */
	private FullDuplexMsgWorker worker;
	/**
	 * JFrame Swing contenant l'interface graphique.
	 */
	private InterfaceGraphique interfaceGraphique;

	/**
	 * 
	 * @param worker
	 *              worker pour la communication TCP avec la serveur.
	 * @param interafaceGraphique
	 *                           JFrame Swing contenant l'interface graphique.
	 */
	public ThreadClient(final FullDuplexMsgWorker worker, final InterfaceGraphique interafaceGraphique) {
		this.worker = worker;
		this.interfaceGraphique = interafaceGraphique;

	}

	@Override
	public void run() {
		ReadMessageStatus status = ReadMessageStatus.ReadUnstarted;

		while (status != ReadMessageStatus.ChannelClosed) { // tant que la connexion fonctionne
			status = worker.readMessage();
			if (status == ReadMessageStatus.ReadDataCompleted) {
				try {
					switch (worker.getMessType()) {
					case MessType.STRINGTYPE:

						String message = (String) worker.getData();
						SwingUtilities.invokeAndWait(new AjouterLigne(this.interfaceGraphique,message, true));

						break;
					case MessType.TRIVIALQUESTIONTYPE:
						TrivialQuestion trivialQuestion = (TrivialQuestion) worker.getData();
						if (LOG_ON && COMM.isEnabledFor(Level.TRACE)) {
							COMM.trace("Question reçue:" + trivialQuestion);
						}
						SwingUtilities.invokeAndWait(new AjouterLigne(this.interfaceGraphique, trivialQuestion.toString(), true));
						this.interfaceGraphique.setReponsePossible(true);
						break;
					case MessType.ANSWERSLISTTYPE:
						AnswersList  answersList = (AnswersList) worker.getData();
						this.interfaceGraphique.setReponsePossible(false);
						SwingUtilities.invokeAndWait(new AjouterLigne(this.interfaceGraphique, answersList.toString(), false));
						break;
					default:
						if (LOG_ON && COMM.isEnabledFor(Level.WARN)) {
							COMM.warn("Type de message non traité: " + worker.getMessType());
						}


					}

				} catch (IOException | InvocationTargetException | InterruptedException e) {
					e.printStackTrace();
				}
			} else {
				if (status == ReadMessageStatus.ChannelClosed) {
					if (LOG_ON && COMM.isEnabledFor(Level.INFO)) {
						COMM.info("Channel closed");
					}
				}
			}
		}

	}

}
